"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPaginationChoices = void 0;
/**
 * Creates Choices for the paginator
 * @param  totalPages  total amount of pages. Array.length is a good way to return this one.
 * @param  currentPage current page. Has to be between [1..totalPages]
 * @return returns the Choices
 */
function createPaginationChoices(totalPages, currentPage) {
    // Numbers have to be within
    // currentPage in [1..totalPages]
    const totalPagesFixed = Math.ceil(totalPages);
    const currentPageFinite = (typeof currentPage === 'number' && Number.isFinite(currentPage)) ? currentPage : 1;
    const currentPageFixed = Math.max(1, Math.min(totalPagesFixed, Math.floor(currentPageFinite)));
    const buttons = {};
    if (!Number.isFinite(totalPagesFixed) || !Number.isFinite(currentPageFixed) || totalPagesFixed < 2) {
        return buttons;
    }
    const before = currentPageFixed - 1;
    const after = currentPageFixed + 1;
    if (currentPageFixed > 1) {
        if (before > 1) {
            buttons[1] = '1 ⏪';
        }
        buttons[before] = `${before} ◀️`;
    }
    buttons[currentPageFixed] = String(currentPageFixed);
    if (currentPageFixed < totalPagesFixed) {
        buttons[after] = `▶️ ${after}`;
        if (after < totalPagesFixed) {
            buttons[totalPagesFixed] = `⏩ ${totalPagesFixed}`;
        }
    }
    return buttons;
}
exports.createPaginationChoices = createPaginationChoices;
//# sourceMappingURL=pagination.js.map