"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureCorrectChoiceKeys = exports.getChoiceTextByKey = exports.getChoiceKeysFromChoices = exports.choicesIsMap = exports.choicesIsArray = void 0;
function choicesIsArray(choices) {
    return Array.isArray(choices);
}
exports.choicesIsArray = choicesIsArray;
function choicesIsMap(choices) {
    return choices instanceof Map;
}
exports.choicesIsMap = choicesIsMap;
function getChoiceKeysFromChoices(choices) {
    if (choicesIsArray(choices)) {
        return choices.map(o => String(o));
    }
    if (choicesIsMap(choices)) {
        return [...choices.keys()].map(o => String(o));
    }
    return Object.keys(choices);
}
exports.getChoiceKeysFromChoices = getChoiceKeysFromChoices;
function getChoiceTextByKey(choices, key) {
    var _a;
    if (choicesIsArray(choices)) {
        return key;
    }
    if (choicesIsMap(choices)) {
        return (_a = choices.get(key)) !== null && _a !== void 0 ? _a : key;
    }
    const choice = choices[key];
    if (choice) {
        return choice;
    }
    return key;
}
exports.getChoiceTextByKey = getChoiceTextByKey;
function ensureCorrectChoiceKeys(actionPrefix, path, choiceKeys) {
    const containSlash = choiceKeys.filter(o => o.includes('/'));
    if (containSlash.length > 0) {
        throw new Error(`Choices can not contain '/'. Found '${containSlash[0]}' in action '${actionPrefix}' at path '${path}'.`);
    }
}
exports.ensureCorrectChoiceKeys = ensureCorrectChoiceKeys;
//# sourceMappingURL=understand-choices.js.map